function [QMPS flag]=CADLIVE_QMPS(delta,h,n_param,n_solution,n_trial)

%
% Kazuhiro Maeda, Hiroyuki Kurata, Quasi-multiparameter sensitivity measure for
% robustness analysis of complex biochemical networks J Theor Biol  272:174 186, 2011
%
global Ctrl;
global Parameter;
%delta=1e-2;  %% MAGNITUDE OF PERTURBATION %%

flag=1;

%standard parameters
StandardParameter = Parameter;

try
    SimResult = CADLIVE_simulation();
    if SimResult.Result == false
        %error('** Cannot Simulate with the Standard Parameter Set! **');
        QMPS=-1;
        flag=-1;
        return;
    end
    
    %stdy: standard values of target functions
    if Ctrl.SOLVER < 10 % Dynamic analysis
        stdy = CADLIVE_setTarget(SimResult.Result_data.y,SimResult.Result_data.time);
    elseif Ctrl.SOLVER == 11 % Steady-state analysis
    	for i=1:length(SimResult.Result_data.sensitivity.QMPS)
        	QMPS.value(i) = SimResult.Result_data.sensitivity.QMPS(i);
        end
        return;
    end

catch
    QMPS=-1;
    flag=-1;
    return;
end
%fprintf('stdy(1)=%f\n',stdy(1));
clear SimResult;



ntar=length(stdy);
QMPS.value=zeros(ntar,1);
% QMPS calculation loop
pFields = fields(Parameter);
lenFields = length(pFields);
a=0;
for i=1:1:lenFields
	name = char(pFields(i));
    if strcmp(name,'constantPlayer')
        continue;
    end
	lenParam = length(Parameter.(name));
    for j=1:1:lenParam
        a=a+1;
        % Report current estimate in the waitbar's message field
    	waitbar(n_trial/n_solution,h,sprintf('%d/%d(%d/%d)',n_trial,n_solution,a,n_param));
    	
        if Parameter.(name)(j).value == 0
    		continue;
    	end
		Parameter.(name)(j).value=Parameter.(name)(j).value*(1+delta);
        
        try
            SimResult = CADLIVE_simulation();
            if SimResult.Result == false
                %error('** Cannot Simulate with the Standard Parameter Set! **');
                QMPS=-1;
                flag=-2;
                return;
            end
    
            %ptby: post-changed values of target functions
            if Ctrl.SOLVER < 10 % Dynamic analysis
                ptby = CADLIVE_setTarget(SimResult.Result_data.y,SimResult.Result_data.time);
            end

        catch
            QMPS=-1;
            flag=-2;
            return;
        end
    	clear SimResult;
        
        %fprintf('%s(%d): p=%f\ts=%f\n',name,j,Parameter.(name)(j).value,StandardParameter.(name)(j).value);
    	for k=1:ntar
    		QMPS.sensitivity(k,1).value(j,1) = (log(ptby(k))-log(stdy(k)))/(log(Parameter.(name)(j).value)-log(StandardParameter.(name)(j).value));
    		QMPS.value(k,1)=QMPS.value(k)+QMPS.sensitivity(k,1).value(j,1)^2;
            %fprintf('ptby=%e\tstdy=%e\tQMPS.value=%f\n',ptby(k),stdy(k),QMPS.value(k));
    	end
    	QMPS.paramName(j,1) = cellstr(sprintf('%s(%d)',name,j));
        
    		
    	%replace standard parameter again
    	Parameter.(name)(j).value = StandardParameter.(name)(j).value;
    end
    
end % QMPS calculation loop
