classdef CADLIVE_SetCtrlO
    properties(Constant)
        Name = 'CADLIVE_SetCtrlO';
        SolTypeAdaptive = 2;
        SolTypeNDF      = 3;
        SolTypeSTD      = 11;
        SolTypeSSystem  = 12;
        DynamicAnalysis = 'Dynamic Analysis';
        Steady_StateAnalysis = 'Steady-State Analysis';
    end
    properties
        VarCount;               % ϐ̑
        AlgebrCount;            % Q̘A̕ϐ̐
        AnalysisType;
        UseSSystem;
        ParamSurvey;
        %SolverType
        SolverType;             % SolverTypeőIl
                                % Runge-Kutta(Equal step-size) : 0
                                % Runge-Kutta(Adaptive step-size) : 1
                                % Runge-Kutta(NDF) : 2
        RkEPS;                  % Relativeɓ͂l
        NdfRtol;                % Relativeɓ͂l
        NdfAtol;                % absoluteɓ͂l
        % Time Span
        StartTime;              % Start time
        EndTime;                % End time
        TimeStep;               % Initial time step-size
        MonitorInterval;        % Monitoring interval
        % NewtonRaphson
        MaxTrialTime;           % Maximum trial times
        ToleranceConvFunc;      % Tolerance for convergence of functions
        ToleranceConvVar;       % Tolerance for convergence of variables
        ParamChgRatio;          % Ratio of changing parameters
        CalcChange;             % Change width for calc. sensitivity(STD)
        % Other
        GValue;                 % G-Value
        YDefValue;              % Y default value
        
        % Y init-data
        Y_initData;             % Y initial data
        
        % EVENT
        Event;                  % Event
        
        % PARAMETER
        Parameter;              % Parameters
    end
    
    methods
        function saveInitial( obj, name )
            fname = strcat( name, '.m' );
            fp_out = fopen(fname,'w');
            
            fprintf( fp_out, 'function [CADLIVE_CTL Y_START]=%s()\n', name );
            fprintf(fp_out,'\n%%=control data\n');
            if ~isnan(obj.VarCount)
                fprintf(fp_out, 'CADLIVE_CTL.N_VAR=%d;\n', obj.VarCount);
            end
            if ~isnan(obj.AlgebrCount)
                fprintf(fp_out, 'CADLIVE_CTL.N_ALGEBR=%d;\n', obj.AlgebrCount);
            end
            fprintf(fp_out,'\n%%==solver\n');
            if ~isnan(obj.SolverType)
                fprintf(fp_out, 'CADLIVE_CTL.SOLVER=%d;\n', obj.SolverType);
            end
            if ~isnan(obj.SolverType)
                fprintf(fp_out, 'CADLIVE_CTL.P_SURVEY=%d;\n', obj.ParamSurvey);
            end
%             if ~isnan(obj.SolverType)
%                 fprintf(fp_out, 'CADLIVE_CTL.PARALLEL=%d;\n', obj.SolverType);
%             end
            if ~isnan(obj.RkEPS)
                fprintf(fp_out, 'CADLIVE_CTL.RK_EPS=%e;\n', obj.RkEPS);
            end
            if ~isnan(obj.NdfRtol)
                fprintf(fp_out, 'CADLIVE_CTL.NDF_RTOL=%e;\n', obj.NdfRtol);
            end
            if ~isnan(obj.NdfAtol)
                fprintf(fp_out, 'CADLIVE_CTL.NDF_ATOL=%e;\n', obj.NdfAtol);
            end
            
            fprintf(fp_out,'\n%%==time span and time step-size\n');
            if ~isnan(obj.StartTime)
                fprintf(fp_out, 'CADLIVE_CTL.T_START=%f;\n', obj.StartTime);
            end
            if ~isnan(obj.EndTime)
                fprintf(fp_out, 'CADLIVE_CTL.T_END=%f;\n', obj.EndTime);
            end
            if ~isnan(obj.TimeStep)
                fprintf(fp_out, 'CADLIVE_CTL.DELTA_T=%f;\n', obj.TimeStep);
            end
            if ~isnan(obj.MonitorInterval)
                fprintf(fp_out, 'CADLIVE_CTL.DELTA_M=%f;\n', obj.MonitorInterval);
            end
            
            fprintf(fp_out,'\n%%==Newton-Raphson Method\n');
            if ~isnan(obj.MaxTrialTime)
                fprintf(fp_out, 'CADLIVE_CTL.NR_TRIAL=%d;\n', obj.MaxTrialTime);
            end
            if ~isnan(obj.ToleranceConvFunc)
                fprintf(fp_out, 'CADLIVE_CTL.NR_TOL_F=%e;\n', obj.ToleranceConvFunc);
            end
            if ~isnan(obj.ToleranceConvVar)
                fprintf(fp_out, 'CADLIVE_CTL.NR_TOL_X=%e;\n', obj.ToleranceConvVar);
            end
            if ~isnan(obj.ParamChgRatio)
                fprintf(fp_out, 'CADLIVE_CTL.NR_RATIO=%f;\n', obj.ParamChgRatio);
            end
            if ~isnan(obj.CalcChange)
                fprintf(fp_out, 'CADLIVE_CTL.NR_SENS_CW=%f;\n', obj.CalcChange);
            end
            
            fprintf(fp_out,'\n%%==Other\n');
            if ~isnan(obj.GValue)
                fprintf(fp_out, 'CADLIVE_CTL.G_VALUE=%f;\n', obj.GValue);
            end
            if ~isnan(obj.YDefValue)
                fprintf(fp_out, 'CADLIVE_CTL.Y_DEFAULT=%e;\n', obj.YDefValue);
            end
            
            [xLength, yLength] = size(obj.Y_initData);
            fprintf(fp_out, '\n%%=Initial parameters\n');
            for j=1:1:yLength
                fprintf(fp_out, 'Y_START(%4d).value = %12.4e;\t%% %s\n', j, obj.Y_initData(j).value, obj.Y_initData(j).tag );
                fprintf(fp_out, 'Y_START(%4d).tag = ''%s'';\n', j, obj.Y_initData(j).tag );
            end
            
            fprintf(fp_out, '\n%%=solver parameter\n');
            fprintf( fp_out, 'CADLIVE_CTL.MASS=eye(CADLIVE_CTL.N_VAR);\n' );
            fprintf( fp_out, 'for i=1:CADLIVE_CTL.N_ALGEBR\n' );
            fprintf( fp_out, '\tCADLIVE_CTL.MASS(i,i)=0;\n' );
            fprintf( fp_out, 'end\n' );
            
            if obj.SolverType==3
            	fprintf(fp_out, '\n%%=ode15s @Events parameter\n');
            	fprintf( fp_out, 'global MAXSIMTIME T0\n' );
            	fprintf( fp_out, 'MAXSIMTIME=5*60;\n' );
            	fprintf( fp_out, 'T0=CADLIVE_CTL.T_START;\n' );
            end
            
            fprintf( fp_out, '\nend\n' );
            fclose(fp_out);
        end
        
        function saveParameter( obj, name )
            fname = strcat( name, '.m' );
            fp_out = fopen(fname,'w');
            
            fprintf( fp_out, 'function [param, event]=%s()\n', name );
            
            fprintf(fp_out,'\n%% T_EVENT\n');
            lenEvents = length(obj.Event);
            for i=1:1:lenEvents
                fprintf(fp_out, '\tevent(%d).name = ''%s'';\n', i, obj.Event(i).name);
                fprintf(fp_out, '\tevent(%d).index = %d;\n', i, obj.Event(i).index);
                if isfield( obj.Event(i), 'time' )
                    if isnan(obj.Event(i).time) == false
                        fprintf(fp_out, '\tevent(%d).time = %f;\n', i, obj.Event(i).time);
                    else
                        fprintf(fp_out, '%%\tevent(%d).time = ;\n', i);
                    end
                else
                    fprintf(fp_out, '%%\tevent(%d).time = ;\n', i);
                end
                fprintf(fp_out, '\tevent(%d).value = %f;\n', i, obj.Event(i).value);
            end
            
            % \̂̃tB[h擾(ł͕ϐ)
        	fprintf(fp_out,'\n%% PARAMETER\n');
            pVars = fieldnames(obj.Parameter);
            lenVars = length(pVars);
            for i=1:1:lenVars
                varName = char(pVars(i));
                
                % \̔z擾(ϐ̒lꗗ)
                var = obj.Parameter.(varName);
                lenValues = length(var);
                for j=1:1:lenValues
                    if isfield( var(j), 'value' ) 
                        if isnan(var(j).value) == false
                            fprintf( fp_out, '\tparam.%s(%d).value = %12.4e;\n', varName, j, var(j).value );
                        else
                            fprintf( fp_out, '%%\tparam.%s(%d).value = is not value;\n', varName, j );
                            %fprintf( fp_out, '\tparam.%s(%d).value = 0.0000e+000;\n', varName, j );
                        end
                    else
                        fprintf( fp_out, '%%\tparam.%s(%d).value = is not value;\n', varName, j );
                        %fprintf( fp_out, '\tparam.%s(%d).value = 0.0000e+000;\n', varName, j );
                    end
                    % val_startɒlΏo͂
                    if isfield(var(j), 'val_start') 
                        if ~isnan(var(j).val_start)
                            fprintf( fp_out, '\tparam.%s(%d).val_start = %d;\n', varName, j, var(j).val_start );
                        end
                    end
                    fprintf( fp_out, '\tparam.%s(%d).num_survey = %d;\n', varName, j, var(j).num_survey );
                    fprintf( fp_out, '\tparam.%s(%d).d_r_s = ''%s'';\n', varName, j, var(j).d_r_s );
                    fprintf( fp_out, '\tparam.%s(%d).tag = ''%s'';\n', varName, j, var(j).tag );
                end
            end
            fprintf( fp_out, 'end\n' );
            fclose(fp_out);
        end
    end
end
