classdef CADLIVE_SsysSTD < CADLIVE_Solver
    %CADLIVE_SSYSSTD S-system̒
    %   S-systemɂ߂B
    
    properties(Access=private)
        INIT_Y;
    end
    
    properties
        % SsysSTĎ
        STDValueFlux;   % STD Values and Fluxes
        LogarythmicGainsMetabolites;    % Logarythmic Gains of Metabolites
        SensitivitiesMetabolitesRateConstants;  % Sensitivities of Metabolites with respect to rate constants
        SensitivitiesMetabolitesKineticOrders;  % Sensitivities of Metabolites with respect to kinetic orders
        LogarythmicGainsFluxes; % Logarythmic Gains of Fluxes
        SensitivitiesFluxesRateConstants;   % Sensitivities of Fluxes with respect to rate constants
        SensitivitiesFluxesKineticOrders;   % Sensitivities of Fluxes with respect to kinetic orders
    end
    
    methods
        function obj = CADLIVE_SsysSTD(ctrl, param, paramT, paramP, init_y)
            % RXgN^
            obj = obj@CADLIVE_Solver(ctrl, param, paramT, paramP);
            obj.INIT_Y = init_y;
        end
        
        function [result, y] = SsysSTD(obj, y)
            % S-systemɂ߂B
            
            n_var = length(y);
            try
            	n_const = length(obj.Param.(CADLIVE_Constant.CONST_PLAYER));
            catch
            	n_const = 0;
            end
            % p[^̃^OG,HZbg
            [Gd, Gi, Hd, Hi] = obj.set_params(n_var, n_const);
            
            % x͂šʂo͂
            [result, y] = obj.std_analysis( Gd, Gi, Hd, Hi, y, n_var, n_const);

            if result == false
                return;
            end
            
            result = true;
            return;
        end
    end
    
    methods(Access=private)
        function [Gd, Gi, Hd, Hi] = set_params(obj, n_var, n_const)
            Gd = zeros(n_var, n_var);
            Gi = zeros(n_var, n_const);
            Hd = zeros(n_var, n_var);
            Hi = zeros(n_var, n_const);
            
            lenG = length(obj.Param.g);
            for i=1:1:lenG
                [res, idx1, idx2] = obj.getParam('g', i);
                if res == true
                    Gd(idx1, idx2) = obj.Param.g(i).value;
                else
                    Gi(idx1, idx2) = obj.Param.g(i).value;
                end
            end
            
            lenH = length(obj.Param.h);
            for i=1:1:lenH
                [res, idx1, idx2] = obj.getParam('h', i);
                if res == true
                    Hd(idx1, idx2) = obj.Param.h(i).value;
                else
                    Hi(idx1, idx2) = obj.Param.h(i).value;
                end
            end
        end
        
        function [res, idx1, idx2] = getParam(obj, valName, i)
            
            tag = obj.Param.(valName)(i).tag;
            tmp = strfind( tag, 'd' );
            res = true;
            if isempty(tmp)
                res = false;
            end
            
            [tmp2, tmp] = strtok(tag, '_');
            [strIdx1, tmp] = strtok(tmp, '_');
            [strIdx2] = strtok(tmp, '_');

            idx1 = str2num(strIdx1);
            idx2 = str2num(strIdx2);
        end

        function [result, y] = std_analysis( obj, Gd, Gi, Hd, Hi, y, n_var, n_const)
            % Ԃɂe͂Ƃ̏o
            
            alpha = zeros( 1, length(obj.Param.kplus) );
            for i=1:1:length(obj.Param.kplus)
                alpha(i) = obj.Param.kplus(i).value;
            end
            beta = zeros( 1, length(obj.Param.kminus) );
            for i=1:1:length(obj.Param.kminus)
                beta(i) = obj.Param.kminus(i).value;
            end
            
            if isfield(obj.Param,'constantPlayer')
            	const_var = zeros( 1, length(obj.Param.constantPlayer) );
            	for i=1:1:length(obj.Param.constantPlayer)
                	const_var(i) = obj.Param.constantPlayer(i).value;
            	end
            end
            
            Ad = Gd - Hd;
            Ai = Gi - Hi;
            [result, Ad, index] = CADLIVE_nr_ludcmp(Ad, n_var);
            if result == false
                return;
            end
            
            % tsFAd_inv ̌vZ (Ƃ肠Â܂܈ڐA)
            Ad_inv = zeros(n_var, n_var);
            col = zeros(1,n_var);
            for i=1:1:n_var
                for j=1:1:n_var
                    col(j) = 0;
                end
                col(i) = 1.0;
                [ col ] = CADLIVE_nr_lubksb( Ad, n_var, index, col );
                
                for j=1:1:n_var
                    Ad_inv(j, i) = col(j);
                end
            end
            Ad1Ai = Ad_inv * Ai;

            % ̌vZ
            for i=1:1:n_var
                tmp = 0;
                for j=1:1:n_var
                    tmp = tmp + Ad_inv(i,j) * log(beta(j)/alpha(j));
                end
                for j=1:1:n_const
                    tmp = tmp - Ad1Ai(i,j) * log(const_var(j));
                end
                y(i) = exp(tmp);
            end
            
            % flux ̌vZ
            % beta ꎞIɃ[ɂfvecvZċ߂B
            kminus_save = obj.Param.kminus;
            for i=1:1:length(obj.Param.kminus)
                obj.Param.kminus(i).value = 0;
            end
            flux = CADLIVE_S_usr_fvec(y, obj.CONTROL_DATA.G_VALUE, obj.Param);
            obj.Param.kminus = kminus_save;
            for i=1:1:n_var;
                obj.STDValueFlux(i).value = y(i) / obj.CONTROL_DATA.G_VALUE;
                obj.STDValueFlux(i).Flux = flux(i);
                obj.STDValueFlux(i).tag = obj.INIT_Y(i).tag;
            end
            GdAd1Ai = Gd * Ad1Ai;
            
            % ϐɑ΂銴x
            for j=1:1:n_const
                for i=1:1:n_var
                    obj.LogarythmicGainsMetabolites(j).constantPlayer(i) = -Ad1Ai(i,j);
                end
            end
            for j=1:1:n_var
                for i=1:1:n_var
                    obj.SensitivitiesMetabolitesRateConstants(j).alpha(i) = -Ad_inv(i,j);
                end
            end

            % G ɊւvZƏo
        	obj.SensitivitiesMetabolitesKineticOrders = obj.sens_kinetic( ...
                obj.SensitivitiesMetabolitesKineticOrders,Gd, Gi, Ad_inv, y, n_var, n_const, true);
            
            % H ɊւvZƏo
        	obj.SensitivitiesMetabolitesKineticOrders = obj.sens_kinetic( ...
                obj.SensitivitiesMetabolitesKineticOrders, Hd, Hi, Ad_inv, y, n_var, n_const, false);
            
            % tbNXɑ΂銴x
            for j=1:1:n_const
                for i=1:1:n_var
                    obj.LogarythmicGainsFluxes(j).constantPlayer(i) = Gi(i,j) - GdAd1Ai(i,j);
                end
            end
            for j=1:1:n_var
                for i=1:1:n_var
                    tmp = 0;
                    if j == i
                        tmp = 1;
                    end
                    for k=1:1:n_var
                        tmp = tmp + Gd(i,k) * (-Ad_inv(k,j));
                    end
                    obj.SensitivitiesFluxesRateConstants(j).alpha(i) = tmp;
                end
            end
            
        	% G ɊւvZƏo
        	obj.SensitivitiesFluxesKineticOrders = obj.sens_kinetic_flux( ...
                obj.SensitivitiesFluxesKineticOrders, Gd, Gi, Ad_inv, y, n_var, n_const, true);
            
        	% H ɊւvZƏo
        	obj.SensitivitiesFluxesKineticOrders = obj.sens_kinetic_flux( ...
                obj.SensitivitiesFluxesKineticOrders, Hd, Hi, Ad_inv, y, n_var, n_const, false);
            
            result = true;
            return;
        end

        function [SMKO] = sens_kinetic( obj, SMKO, GHd, GHi, Ad_inv, y, n_var, n_const, f_isG)
            % kinetic oder ɑ΂Metabolite̊x̌vZƏoB
            [label, my_sign] = obj.getLabelSign(f_isG);
            if isfield(obj.Param,'constantPlayer')
            	const_var = obj.Param.constantPlayer;
            end
            cnt = 0;
            if isfield(SMKO, label)
                cnt = length(SMKO.(label));
            end
             for j=1:1:n_var
                for k=1:1:n_var
                    if GHd(j,k) == 0
                        continue;
                    end
                    cnt = cnt+1;
                    for i=1:1:n_var
                        SMKO.(label)(cnt,i) = my_sign*GHd(j,k) * Ad_inv(i,j) * log(y(k));
                    end
                end
                
                for k=1:1:n_const
                    if GHi(j,k) == 0
                        continue;
                    end
                    cnt = cnt+1;
                    for i=1:1:n_var
                        SMKO.(label)(cnt,i) = my_sign * GHi(j,k) * Ad_inv(i,j) * log(const_var(k).value);
                    end
                end
            end
        end
        
        function SFKO = sens_kinetic_flux(obj, SFKO, GHd, GHi, Ad_inv, y, n_var, n_const, f_isG)
            % kinetic oder ɑ΂Flux̊x̌vZƏo
            [label, my_sign] = obj.getLabelSign(f_isG);
            if isfield(obj.Param, 'constantPlayer')
            	const_var = obj.Param.constantPlayer;
            end
            cnt = 0;
            if isfield(SFKO, label)
                cnt = length(SFKO.(label));
            end
            for j=1:1:n_var
                for k=1:1:n_var
                    if GHd(j,k) == 0
                        continue;
                    end
                    cnt = cnt + 1;
                    for i=1:1:n_var
                        val = 0;
                        if i == j 
                            val = GHd(j,k) * log(y(k));
                        end
                        for m=1:1:n_var
                            val = val + GHd(i,m) * my_sign * GHd(j,k) * Ad_inv(m,j) * log(y(k));
                        end
                        SFKO.(label)(cnt,i) = val;
                    end
                end
                
                for k=1:1:n_const
                    if GHi(j,k) == 0
                        continue;
                    end
                    cnt = cnt + 1;
                    for i=1:1:n_var
                        val = 0;
                        if i == j
                            val = GHi(j,k) * log(const_var(k).value);
                        end
                        for m=1:1:n_var
                            val = val + GHd(i,m) * my_sign * GHi(j,k) * Ad_inv(m,j) * log(const_var(k).value);
                        end
                        SFKO.(label)(cnt,i) = val;
                    end
                end
            end
        end
        
        function [label, sign] = getLabelSign(obj, f_isG)
            label = 'h';
            sign = 1;
            if f_isG
                label = 'g';
                sign = -1;
            end
        end

    end
    
end

