%function for TPS to QMPS
%delta: ratio of a change for parameters
%QMPS: values of QMPS for results of each TPS
%tag: tags of target functions
function [QMPS, tag]=CADLIVE_TPStoQMPS(delta, info)

global Ctrl;
global Parameter;
global Events;

clear CADLIVE_param;

[Parameter, Events] = CADLIVE_param();
%initial values
[Ctrl, init_y] = CADLIVE_initial();


load(info.tpsFile)

n_gene=length(searchParam);
n_solution = length(solution);

pFields = fields(Parameter);
lenFields = length(pFields);
    
n_param = 0;
for i=1:lenFields
	name = char(pFields(i));
    if strcmp(name,'constantPlayer')
        continue;
    end
    lenParam = length(Parameter.(name));
    n_param = n_param + lenParam;
end
    
%progress bar
h = waitbar(0,sprintf('0/%d(1/%d)',n_solution,n_param),'Name','TPStoQMPS Progress',...
            'CreateCancelBtn',...
            'setappdata(gcbf,''canceling'',1)');
setappdata(h,'canceling',0);
waitbar(0/n_solution,h,sprintf('0/%d(1/%d)',n_solution,n_param));

nQMPS=0;
for i=1:n_solution
   	% Check for Cancel button press
    if getappdata(h,'canceling')
    	delete(h)       % DELETE the waitbar;
    	[nQMPS2, ntag]=size(QMPS);
		tag=cell(ntag,1);
		for i=1:ntag
    		tag(i)=cellstr(sprintf('target%d',i));
		end
        return;
    end

    
	for j=1:n_gene
		name=char(searchParam(j).paramName);
		ind=searchParam(j).paramIndex;
		Parameter.(name)(ind).value=solution(i).paramValue(j);
	end
	
	[QMPS0, flag]=CADLIVE_QMPS(delta,h,n_param,n_solution,i-1);
    %fprintf('flag=%d\n',flag);
    if flag == 1
        nQMPS=nQMPS+1;
        QMPS(nQMPS,:)=QMPS0.value';
    end
    
    % Report current estimate in the waitbar's message field
    waitbar(i/n_solution,h,sprintf('%d/%d(1/%d)',i,n_solution,n_param));
end
delete(h)       % DELETE the waitbar;

if nQMPS ~= n_solution
	fprintf('The %d simulations cannot be solved.\n',(n_solution-nQMPS));
end

%fprintf('nQMPS=%d\n',nQMPS);

[nQMPS2, ntag]=size(QMPS);
tag=cell(ntag,1);
for i=1:ntag
	if Ctrl.SOLVER == 11
		tag(i) = cellstr(init_y(i).tag);
	else
    	tag(i)=cellstr(sprintf('target%d',i));
    end
end


