function [solution, searchParam, info, flag] = CADLIVE_myTPS(info)
global Parameter;
global Events;

%Iteration limit for being not able to find solution
limit = 10000;


info.n_trial=0;
info.n_solution=0;
info.n_rs=0;
info.n_coarse_solution=0;
info.n_solution_obtained_in_GA=0;
info.n_solution_obtained_in_first_generation=0;
info.n_evaluation=0;

info.progressStop=false;

%Call initial parameters
[Parameter, Events] = CADLIVE_param();

[searchParam, flag]=CADLIVE_setSearchParamter(info.searchType);
info.n_gene=length(searchParam);

%Error serach parameter setting
if flag == -1
	solution = -1;
	searchParam = -1;
    info = -1;
	return;
end

%Initial setting for progress bar
h = waitbar(0,sprintf('0/%d',info.term_n_solution),'Name','TPS Progress',...
            'CreateCancelBtn',...
            'setappdata(gcbf,''canceling'',1)');
setappdata(h,'canceling',0);
info.h = h;

solution(1).fitness=1e+20;
%start TPS
while info.n_solution < info.term_n_solution
	
	info = CADLIVE_showProgressBar(info);
   	% Check for Cancel button press
    if info.progressStop
    	if info.n_solution==0
    		solution = -1;
			searchParam = -1;
    		info = -1;
    		flag=-1;
    	else
    		solution = CADLIVE_sortSolution(info, solution);
		end
        delete(h)       % DELETE the waitbar;
        return;
    end
    
    if info.n_evaluation > limit && info.n_solution==0
    	solution = -1;
		searchParam = -1;
    	info = -1;
    	flag=-1;
    	delete(h)       % DELETE the waitbar;
    	errordlg( ...
        		sprintf('Cannot search suitable parameters. (search limit= %d)',limit) ...
        		, 'TPS error' ...
        		);
        return;
    end
    
	%Random search
	population0 = CADLIVE_randomSearch(searchParam, info);
	info.n_evaluation = info.n_evaluation + 1;
	info.n_trial = info.n_trial + 1;
	%fprintf('RS:fitness=%e\n',population0.fitness);
	
	%Show evaluation value if not find solution at all
	if info.n_solution == 0 && mod(info.n_evaluation,100)==0
		if population0.fitness == 1e+10
			fprintf('EVA=%d RS:Cannot solve.\n', info.n_evaluation);
		else
			fprintf('EVA=%d RS:fitness=%e.\n', info.n_evaluation, population0.fitness);
		end
	end
	
	%GA
	if population0.fitness <= info.AE
		info.n_solution = info.n_solution + 1;
		info.n_rs = info.n_rs + 1;
		
		solution = CADLIVE_saveTPSResults(solution, info, population0, searchParam, 'RS');
		
        info = CADLIVE_showProgressBar(info);
    	
	elseif population0.fitness <= info.AE*info.AEC
		%start GA
		info.n_coarse_solution = info.n_coarse_solution + 1;
		info.n_evaluation = info.n_evaluation + info.n_population;
		
		populations = CADLIVE_TPSgetInitialPopulation(population0, searchParam, info);
		
		%fprintf('GAinit:fitness=%e\n',population(1).fitness);
		if populations(1).fitness <= info.AE
    		info.n_solution_obtained_in_GA = info.n_solution_obtained_in_GA + 1;
    		info.n_solution_obtained_in_first_generation = info.n_solution_obtained_in_first_generation + 1;
    		info.n_solution = info.n_solution + 1;
    		
    		solution = CADLIVE_saveTPSResults(solution, info, populations(1), searchParam, 'GAinit');
			
            info = CADLIVE_showProgressBar(info);
    	else
			for i = 2:info.n_generation
			
				info = CADLIVE_showProgressBar(info);
			   	% Check for Cancel button press
    			if info.progressStop
    				if info.n_solution==0
    					solution = -1;
						searchParam = -1;
    					info = -1;
    					flag=-1;
    				else
    					solution = CADLIVE_sortSolution(info, solution);
					end
    				
        			delete(h)       % DELETE the waitbar;
        			return;
    			end
    			[populations, info] = CADLIVE_MGGvariant(populations, searchParam, info);
    			
    			info.n_evaluation = info.n_evaluation + info.n_population;
    			%fprintf('GA%d:fitness=%e\n',i,population(1).fitness);
    			if populations(1).fitness <= info.AE
        			break;
    			end
			end

			if populations(1).fitness <= info.AE
				info.n_solution_obtained_in_GA = info.n_solution_obtained_in_GA + 1;
				info.n_solution = info.n_solution + 1;
				
				solution = CADLIVE_saveTPSResults(solution, info, populations(1), searchParam, sprintf('GA_%d',i));

                info = CADLIVE_showProgressBar(info);
			end
		end
		clear populations
	end
	
	clear population0

end

solution = CADLIVE_sortSolution(info, solution);

delete(h)       % DELETE the waitbar;
