function [C F] = CADLIVE_myhist(X,n,lb,ub)
% [C F] = CADLIVE_myhist(X,n,lb,ub)
%
% myhist function makes histograms from X.
% To plot the output, use "plot(C,F./length(X),'-o');".
%
% X:  Data matrix whose each column shows each data set.
% n:  The number of classes.
% lb: Lower bound of the histograms. 
% ub: Upper bound of the histograms. 
% C:  Class value vector.
% F:  Frequency matrix whose column shows one data set.


% Error Handling
if lb>=ub
    error('ub must be greater than lb.')
end
if mod(n,1)~=0 || n<=0
    error('n must be a positive integer.')
end

% Make Histogram
[n_row n_col]=size(X);
F=zeros(n,n_col);
d=(ub-lb)/n;

for i=1:n_row
    for j=1:n_col
        for k=1:n
            if lb+(k-1)*d<=X(i,j) && X(i,j)<=lb+k*d
                F(k,j)=F(k,j)+1;
                break;
            end
        end
    end
end

C=zeros(n,1);
for k=1:n
    C(k)=lb+(k-0.5)*d;
end

