function [SimResult fitness] = CADLIVE_optimtool(options)

global Ctrl;
global Parameter;
global Events;
global searchType;
global searchParam;

if nargin ~= 1
    options = gaoptimset(@ga);
end

searchType = 'log';

[Ctrl, init_y] = CADLIVE_initial();
[Parameter, Events] = CADLIVE_param();
[searchParam flag]=CADLIVE_setSearchParamter(searchType);
%search paramters are not set
if flag == -1
	error('Search parameters are not set.');
end

%optimization type
[x fitness] = ga(@CADLIVE_optimtoolfunc,length(searchParam),[],[],[],[],[],[],[],options);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

for i=1:length(searchParam)
    individual.gene(i) = x(i);
end
searchParam = CADLIVE_decodeGene2Variable(individual,length(searchParam),searchParam, searchType);
CADLIVE_changeParam(searchParam);


%Simulation with optimized parameters
try

	SimResult = CADLIVE_simulation();
	if SimResult.Result == false
    	errordlg( ...
        	'cannot optimize.' ...
        	, 'Optimization error' ...
        );
    	return;
	end

catch err
    fprintf('%s\n', err.identifier);
    fprintf('%s\n', err.message);
    fprintf('stack trace\n');
    lenStack = length(err.stack);
    for i=1:1:lenStack
    	fprintf( '%04d:%s \n', i, err.stack(i).file );
        fprintf( '     %s(line:%d)\n', err.stack(i).name, err.stack(i).line );
    end
    msgbox( CADLIVE_Messages.FAILE_SIMULATION );
    return;
end

% Output the optimized parameters
fo=fopen('CADLIVE_OptParam.m','w');
fprintf(fo,'function [param, event]=CADLIVE_param()\n');
fprintf(fo,'load OptimizedParamter.mat;\n');
fclose(fo);

param = Parameter;
event = Events;
save('OptimizedParamter.mat','param','event');
